@@ -81,7 +81,7 @@ QT_BEGIN_NAMESPACE
 namespace QtPrivate {
 Q_CORE_EXPORT bool isBuiltinType(const QByteArray &type)
 {
-    int id = QMetaType::type(type);
+    int id = QMetaType::fromName(type).id();
     if (!id && !type.isEmpty() && type != "void")
         return false;
     return (id < QMetaType::User);
@@ -89,7 +89,7 @@ Q_CORE_EXPORT bool isBuiltinType(const QByteArray &type)
 } // namespace QtPrivate
 
 // copied from qmetaobject.cpp
-static inline Q_DECL_UNUSED const QMetaObjectPrivate *priv(const uint* data)
+[[maybe_unused]] static inline const QMetaObjectPrivate *priv(const uint* data)
 { return reinterpret_cast<const QMetaObjectPrivate*>(data); }
 
 class QMetaMethodBuilderPrivate
@@ -146,7 +146,7 @@ public:
         return signature.left(qMax(signature.indexOf('('), 0));
     }
 };
-Q_DECLARE_TYPEINFO(QMetaMethodBuilderPrivate, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QMetaMethodBuilderPrivate, Q_RELOCATABLE_TYPE);
 
 class QMetaPropertyBuilderPrivate
 {
@@ -156,13 +156,10 @@ public:
              int _revision = 0)
         : name(_name),
           type(QMetaObject::normalizedType(_type.constData())),
-          flags(Readable | Writable | Scriptable), notifySignal(-1),
+          flags(Readable | Writable | Scriptable), notifySignal(notifierIdx),
           revision(_revision)
     {
-        if (notifierIdx >= 0) {
-            flags |= Notify;
-            notifySignal = notifierIdx;
-        }
+
     }
 
     QByteArray name;
@@ -184,12 +181,12 @@ public:
             flags &= ~f;
     }
 };
-Q_DECLARE_TYPEINFO(QMetaPropertyBuilderPrivate, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QMetaPropertyBuilderPrivate, Q_RELOCATABLE_TYPE);
 
 class QMetaEnumBuilderPrivate
 {
 public:
-    QMetaEnumBuilderPrivate(const QByteArray& _name)
+    QMetaEnumBuilderPrivate(const QByteArray &_name)
         : name(_name), enumName(_name), isFlag(false), isScoped(false)
     {
     }
@@ -199,9 +196,9 @@ public:
     bool isFlag;
     bool isScoped;
     QList<QByteArray> keys;
-    QVector<int> values;
+    QList<int> values;
 };
-Q_DECLARE_TYPEINFO(QMetaEnumBuilderPrivate, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QMetaEnumBuilderPrivate, Q_RELOCATABLE_TYPE);
 
 class QMetaObjectBuilderPrivate
 {
@@ -210,10 +207,9 @@ public:
         : flags(0)
     {
         superClass = &QObject::staticMetaObject;
-        staticMetacallFunction = 0;
+        staticMetacallFunction = nullptr;
     }
 
-    bool hasRevisionedProperties() const;
     bool hasRevisionedMethods() const;
 
     QByteArray className;
@@ -229,15 +225,6 @@ public:
     int flags;
 };
 
-bool QMetaObjectBuilderPrivate::hasRevisionedProperties() const
-{
-    for (const auto &property : properties) {
-        if (property.revision)
-            return true;
-    }
-    return false;
-}
-
 bool QMetaObjectBuilderPrivate::hasRevisionedMethods() const
 {
     for (const auto &method : methods) {
@@ -266,8 +253,8 @@ QMetaObjectBuilder::QMetaObjectBuilder()
 
     \sa addMetaObject()
 */
-QMetaObjectBuilder::QMetaObjectBuilder
-    (const QMetaObject *prototype, QMetaObjectBuilder::AddMembers members)
+QMetaObjectBuilder::QMetaObjectBuilder(const QMetaObject *prototype,
+                                       QMetaObjectBuilder::AddMembers members)
 {
     d = new QMetaObjectBuilderPrivate();
     addMetaObject(prototype, members);
@@ -298,7 +285,7 @@ QByteArray QMetaObjectBuilder::className() const
 
     \sa className(), setSuperClass()
 */
-void QMetaObjectBuilder::setClassName(const QByteArray& name)
+void QMetaObjectBuilder::setClassName(const QByteArray &name)
 {
     d->className = name;
 }
@@ -334,9 +321,9 @@ void QMetaObjectBuilder::setSuperClass(const QMetaObject *meta)
 
     \sa setFlags()
 */
-QMetaObjectBuilder::MetaObjectFlags QMetaObjectBuilder::flags() const
+MetaObjectFlags QMetaObjectBuilder::flags() const
 {
-    return (QMetaObjectBuilder::MetaObjectFlags)d->flags;
+    return MetaObjectFlags(d->flags);
 }
 
 /*!
@@ -431,7 +418,7 @@ int QMetaObjectBuilder::relatedMetaObjectCount() const
 
     \sa method(), methodCount(), removeMethod(), indexOfMethod()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QByteArray& signature)
+QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QByteArray &signature)
 {
     int index = int(d->methods.size());
     d->methods.push_back(QMetaMethodBuilderPrivate(QMetaMethod::Method, signature));
@@ -447,12 +434,11 @@ QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QByteArray& signature)
 
     \sa method(), methodCount(), removeMethod(), indexOfMethod()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addMethod
-    (const QByteArray& signature, const QByteArray& returnType)
+QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QByteArray &signature,
+                                                 const QByteArray &returnType)
 {
     int index = int(d->methods.size());
-    d->methods.push_back(QMetaMethodBuilderPrivate
-        (QMetaMethod::Method, signature, returnType));
+    d->methods.push_back(QMetaMethodBuilderPrivate(QMetaMethod::Method, signature, returnType));
     return QMetaMethodBuilder(this, index);
 }
 
@@ -467,7 +453,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addMethod
 
     \sa method(), methodCount(), removeMethod(), indexOfMethod()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QMetaMethod& prototype)
+QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QMetaMethod &prototype)
 {
     QMetaMethodBuilder method;
     if (prototype.methodType() == QMetaMethod::Method)
@@ -495,7 +481,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addMethod(const QMetaMethod& prototype)
 
     \sa addMethod(), addSignal(), indexOfSlot()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addSlot(const QByteArray& signature)
+QMetaMethodBuilder QMetaObjectBuilder::addSlot(const QByteArray &signature)
 {
     int index = int(d->methods.size());
     d->methods.push_back(QMetaMethodBuilderPrivate(QMetaMethod::Slot, signature));
@@ -510,11 +496,11 @@ QMetaMethodBuilder QMetaObjectBuilder::addSlot(const QByteArray& signature)
 
     \sa addMethod(), addSlot(), indexOfSignal()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addSignal(const QByteArray& signature)
+QMetaMethodBuilder QMetaObjectBuilder::addSignal(const QByteArray &signature)
 {
     int index = int(d->methods.size());
-    d->methods.push_back(QMetaMethodBuilderPrivate
-        (QMetaMethod::Signal, signature, QByteArray("void"), QMetaMethod::Public));
+    d->methods.push_back(QMetaMethodBuilderPrivate(QMetaMethod::Signal, signature,
+                                                   QByteArray("void"), QMetaMethod::Public));
     return QMetaMethodBuilder(this, index);
 }
 
@@ -527,7 +513,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addSignal(const QByteArray& signature)
     \sa constructor(), constructorCount(), removeConstructor()
     \sa indexOfConstructor()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QByteArray& signature)
+QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QByteArray &signature)
 {
     int index = int(d->constructors.size());
     d->constructors.push_back(QMetaMethodBuilderPrivate(QMetaMethod::Constructor, signature,
@@ -546,7 +532,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QByteArray& signatur
     \sa constructor(), constructorCount(), removeConstructor()
     \sa indexOfConstructor()
 */
-QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QMetaMethod& prototype)
+QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QMetaMethod &prototype)
 {
     Q_ASSERT(prototype.methodType() == QMetaMethod::Constructor);
     QMetaMethodBuilder ctor = addConstructor(prototype.methodSignature());
@@ -567,8 +553,8 @@ QMetaMethodBuilder QMetaObjectBuilder::addConstructor(const QMetaMethod& prototy
 
     \sa property(), propertyCount(), removeProperty(), indexOfProperty()
 */
-QMetaPropertyBuilder QMetaObjectBuilder::addProperty
-    (const QByteArray& name, const QByteArray& type, int notifierId)
+QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QByteArray &name, const QByteArray &type,
+                                                     int notifierId)
 {
     int index = int(d->properties.size());
     d->properties.push_back(QMetaPropertyBuilderPrivate(name, type, notifierId));
@@ -583,7 +569,7 @@ QMetaPropertyBuilder QMetaObjectBuilder::addProperty
 
     \sa property(), propertyCount(), removeProperty(), indexOfProperty()
 */
-QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QMetaProperty& prototype)
+QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QMetaProperty &prototype)
 {
     QMetaPropertyBuilder property = addProperty(prototype.name(), prototype.typeName());
     property.setReadable(prototype.isReadable());
@@ -592,7 +578,6 @@ QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QMetaProperty& protot
     property.setDesignable(prototype.isDesignable());
     property.setScriptable(prototype.isScriptable());
     property.setStored(prototype.isStored());
-    property.setEditable(prototype.isEditable());
     property.setUser(prototype.isUser());
     property.setStdCppSet(prototype.hasStdCppSet());
     property.setEnumOrFlag(prototype.isEnumType());
@@ -606,7 +591,6 @@ QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QMetaProperty& protot
         if (index == -1)
             index = addMethod(method).index();
         d->properties[property._index].notifySignal = index;
-        d->properties[property._index].setFlag(Notify, true);
     }
     return property;
 }
@@ -619,7 +603,7 @@ QMetaPropertyBuilder QMetaObjectBuilder::addProperty(const QMetaProperty& protot
     \sa enumerator(), enumeratorCount(), removeEnumerator()
     \sa indexOfEnumerator()
 */
-QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QByteArray& name)
+QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QByteArray &name)
 {
     int index = int(d->enumerators.size());
     d->enumerators.push_back(QMetaEnumBuilderPrivate(name));
@@ -635,7 +619,7 @@ QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QByteArray& name)
     \sa enumerator(), enumeratorCount(), removeEnumerator()
     \sa indexOfEnumerator()
 */
-QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QMetaEnum& prototype)
+QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QMetaEnum &prototype)
 {
     QMetaEnumBuilder en = addEnumerator(prototype.name());
     en.setEnumName(prototype.enumName());
@@ -654,7 +638,7 @@ QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QMetaEnum& prototype)
     \sa classInfoCount(), classInfoName(), classInfoValue(), removeClassInfo()
     \sa indexOfClassInfo()
 */
-int QMetaObjectBuilder::addClassInfo(const QByteArray& name, const QByteArray& value)
+int QMetaObjectBuilder::addClassInfo(const QByteArray &name, const QByteArray &value)
 {
     int index = d->classInfoNames.size();
     d->classInfoNames += name;
@@ -688,8 +672,8 @@ int QMetaObjectBuilder::addRelatedMetaObject(const QMetaObject *meta)
     The \a members parameter indicates which members of \a prototype
     should be added.  The default is AllMembers.
 */
-void QMetaObjectBuilder::addMetaObject
-        (const QMetaObject *prototype, QMetaObjectBuilder::AddMembers members)
+void QMetaObjectBuilder::addMetaObject(const QMetaObject *prototype,
+                                       QMetaObjectBuilder::AddMembers members)
 {
     Q_ASSERT(prototype);
     int index;
@@ -749,7 +733,7 @@ void QMetaObjectBuilder::addMetaObject
         Q_ASSERT(priv(prototype->d.data)->revision >= 2);
         const auto *objects = prototype->d.relatedMetaObjects;
         if (objects) {
-            while (*objects != 0) {
+            while (*objects != nullptr) {
                 addRelatedMetaObject(*objects);
                 ++objects;
             }
@@ -831,7 +815,7 @@ const QMetaObject *QMetaObjectBuilder::relatedMetaObject(int index) const
     if (index >= 0 && index < d->relatedMetaObjects.size())
         return d->relatedMetaObjects[index];
     else
-        return 0;
+        return nullptr;
 }
 
 /*!
@@ -880,7 +864,6 @@ void QMetaObjectBuilder::removeMethod(int index)
             // Adjust the indices of property notify signal references.
             if (property.notifySignal == index) {
                 property.notifySignal = -1;
-                property.setFlag(Notify, false);
             } else if (property.notifySignal > index)
                 property.notifySignal--;
         }
@@ -964,7 +947,7 @@ void QMetaObjectBuilder::removeRelatedMetaObject(int index)
 
     \sa method(), methodCount(), addMethod(), removeMethod()
 */
-int QMetaObjectBuilder::indexOfMethod(const QByteArray& signature)
+int QMetaObjectBuilder::indexOfMethod(const QByteArray &signature)
 {
     QByteArray sig = QMetaObject::normalizedSignature(signature);
     for (const auto &method : d->methods) {
@@ -980,7 +963,7 @@ int QMetaObjectBuilder::indexOfMethod(const QByteArray& signature)
 
     \sa indexOfMethod(), indexOfSlot()
 */
-int QMetaObjectBuilder::indexOfSignal(const QByteArray& signature)
+int QMetaObjectBuilder::indexOfSignal(const QByteArray &signature)
 {
     QByteArray sig = QMetaObject::normalizedSignature(signature);
     for (const auto &method : d->methods) {
@@ -996,7 +979,7 @@ int QMetaObjectBuilder::indexOfSignal(const QByteArray& signature)
 
     \sa indexOfMethod(), indexOfSignal()
 */
-int QMetaObjectBuilder::indexOfSlot(const QByteArray& signature)
+int QMetaObjectBuilder::indexOfSlot(const QByteArray &signature)
 {
     QByteArray sig = QMetaObject::normalizedSignature(signature);
     for (const auto &method : d->methods) {
@@ -1012,7 +995,7 @@ int QMetaObjectBuilder::indexOfSlot(const QByteArray& signature)
 
     \sa constructor(), constructorCount(), addConstructor(), removeConstructor()
 */
-int QMetaObjectBuilder::indexOfConstructor(const QByteArray& signature)
+int QMetaObjectBuilder::indexOfConstructor(const QByteArray &signature)
 {
     QByteArray sig = QMetaObject::normalizedSignature(signature);
     for (const auto &constructor : d->constructors) {
@@ -1028,7 +1011,7 @@ int QMetaObjectBuilder::indexOfConstructor(const QByteArray& signature)
 
     \sa property(), propertyCount(), addProperty(), removeProperty()
 */
-int QMetaObjectBuilder::indexOfProperty(const QByteArray& name)
+int QMetaObjectBuilder::indexOfProperty(const QByteArray &name)
 {
     for (const auto &property : d->properties) {
         if (name == property.name)
@@ -1043,7 +1026,7 @@ int QMetaObjectBuilder::indexOfProperty(const QByteArray& name)
 
     \sa enumertor(), enumeratorCount(), addEnumerator(), removeEnumerator()
 */
-int QMetaObjectBuilder::indexOfEnumerator(const QByteArray& name)
+int QMetaObjectBuilder::indexOfEnumerator(const QByteArray &name)
 {
     for (const auto &enumerator : d->enumerators) {
         if (name == enumerator.name)
@@ -1059,7 +1042,7 @@ int QMetaObjectBuilder::indexOfEnumerator(const QByteArray& name)
     \sa classInfoName(), classInfoValue(), classInfoCount(), addClassInfo()
     \sa removeClassInfo()
 */
-int QMetaObjectBuilder::indexOfClassInfo(const QByteArray& name)
+int QMetaObjectBuilder::indexOfClassInfo(const QByteArray &name)
 {
     for (int index = 0; index < d->classInfoNames.size(); ++index) {
         if (name == d->classInfoNames[index])
@@ -1103,13 +1086,13 @@ int QMetaStringTable::enter(const QByteArray &value)
 
 int QMetaStringTable::preferredAlignment()
 {
-    return Q_ALIGNOF(QByteArrayData);
+    return alignof(uint);
 }
 
 // Returns the size (in bytes) required for serializing this string table.
 int QMetaStringTable::blobSize() const
 {
-    int size = m_entries.size() * sizeof(QByteArrayData);
+    int size = int(m_entries.size() * 2 * sizeof(uint));
     Entries::const_iterator it;
     for (it = m_entries.constBegin(); it != m_entries.constEnd(); ++it)
         size += it.key().size() + 1;
@@ -1120,14 +1103,12 @@ static void writeString(char *out, int i, const QByteArray &str,
                         const int offsetOfStringdataMember, int &stringdataOffset)
 {
     int size = str.size();
-    qptrdiff offset = offsetOfStringdataMember + stringdataOffset
-            - i * sizeof(QByteArrayData);
-    const QByteArrayData data =
-        Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(size, offset);
+    int offset = offsetOfStringdataMember + stringdataOffset;
+    uint offsetLen[2] = { uint(offset), uint(size) };
 
-    memcpy(out + i * sizeof(QByteArrayData), &data, sizeof(QByteArrayData));
+    memcpy(out + 2 * i * sizeof(uint), &offsetLen, 2 * sizeof(uint));
 
-    memcpy(out + offsetOfStringdataMember + stringdataOffset, str.constData(), size);
+    memcpy(out + offset, str.constData(), size);
     out[offsetOfStringdataMember + stringdataOffset + size] = '\0';
 
     stringdataOffset += size + 1;
@@ -1139,13 +1120,13 @@ static void writeString(char *out, int i, const QByteArray &str,
 // moc (see generator.cpp).
 void QMetaStringTable::writeBlob(char *out) const
 {
-    Q_ASSERT(!(reinterpret_cast<quintptr>(out) & (preferredAlignment()-1)));
+    Q_ASSERT(!(reinterpret_cast<quintptr>(out) & (preferredAlignment() - 1)));
 
-    int offsetOfStringdataMember = m_entries.size() * sizeof(QByteArrayData);
+    int offsetOfStringdataMember = int(m_entries.size() * 2 * sizeof(uint));
     int stringdataOffset = 0;
 
     // qt_metacast expects the first string in the string table to be the class name.
-    writeString(out, /*index*/0, m_className, offsetOfStringdataMember, stringdataOffset);
+    writeString(out, /*index*/ 0, m_className, offsetOfStringdataMember, stringdataOffset);
 
     for (Entries::ConstIterator it = m_entries.constBegin(), end = m_entries.constEnd();
          it != end; ++it) {
@@ -1183,8 +1164,6 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     int enumIndex;
     int index;
     bool hasRevisionedMethods = d->hasRevisionedMethods();
-    bool hasRevisionedProperties = d->hasRevisionedProperties();
-    bool hasNotifySignals = false;
 
     if (relocatable &&
         (d->relatedMetaObjects.size() > 0 || d->staticMetacallFunction))
@@ -1196,8 +1175,9 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     ALIGN(size, int);
     if (buf) {
         if (!relocatable) meta->d.superdata = d->superClass;
-        meta->d.relatedMetaObjects = 0;
-        meta->d.extradata = 0;
+        meta->d.relatedMetaObjects = nullptr;
+        meta->d.extradata = nullptr;
+        meta->d.metaTypes = nullptr;
         meta->d.static_metacall = d->staticMetacallFunction;
     }
 
@@ -1206,19 +1186,13 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         = reinterpret_cast<QMetaObjectPrivate *>(buf + size);
     int pmetaSize = size;
     dataIndex = MetaObjectPrivateFieldCount;
-    for (const auto &property : d->properties) {
-        if (property.notifySignal != -1) {
-            hasNotifySignals = true;
-            break;
-        }
-    }
     int methodParametersDataSize =
             ((aggregateParameterCount(d->methods)
              + aggregateParameterCount(d->constructors)) * 2) // types and parameter names
             - int(d->methods.size())       // return "parameters" don't have names
             - int(d->constructors.size()); // "this" parameters don't have names
     if (buf) {
-        Q_STATIC_ASSERT_X(QMetaObjectPrivate::OutputRevision == 8, "QMetaObjectBuilder should generate the same version as moc");
+        static_assert(QMetaObjectPrivate::OutputRevision == 9, "QMetaObjectBuilder should generate the same version as moc");
         pmeta->revision = QMetaObjectPrivate::OutputRevision;
         pmeta->flags = d->flags;
         pmeta->className = 0;   // Class name is always the first string.
@@ -1230,7 +1204,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
         pmeta->methodCount = int(d->methods.size());
         pmeta->methodData = dataIndex;
-        dataIndex += 5 * int(d->methods.size());
+        dataIndex += QMetaObjectPrivate::IntsPerMethod * int(d->methods.size());
         if (hasRevisionedMethods)
             dataIndex += int(d->methods.size());
         paramsIndex = dataIndex;
@@ -1238,33 +1212,25 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
         pmeta->propertyCount = int(d->properties.size());
         pmeta->propertyData = dataIndex;
-        dataIndex += 3 * int(d->properties.size());
-        if (hasNotifySignals)
-            dataIndex += int(d->properties.size());
-        if (hasRevisionedProperties)
-            dataIndex += int(d->properties.size());
+        dataIndex += QMetaObjectPrivate::IntsPerProperty * int(d->properties.size());
 
         pmeta->enumeratorCount = int(d->enumerators.size());
         pmeta->enumeratorData = dataIndex;
-        dataIndex += 5 * int(d->enumerators.size());
+        dataIndex += QMetaObjectPrivate::IntsPerEnum * int(d->enumerators.size());
 
         pmeta->constructorCount = int(d->constructors.size());
         pmeta->constructorData = dataIndex;
-        dataIndex += 5 * int(d->constructors.size());
+        dataIndex += QMetaObjectPrivate::IntsPerMethod * int(d->constructors.size());
     } else {
         dataIndex += 2 * int(d->classInfoNames.size());
-        dataIndex += 5 * int(d->methods.size());
+        dataIndex += QMetaObjectPrivate::IntsPerMethod * int(d->methods.size());
         if (hasRevisionedMethods)
             dataIndex += int(d->methods.size());
         paramsIndex = dataIndex;
         dataIndex += methodParametersDataSize;
-        dataIndex += 3 * int(d->properties.size());
-        if (hasNotifySignals)
-            dataIndex += int(d->properties.size());
-        if (hasRevisionedProperties)
-            dataIndex += int(d->properties.size());
-        dataIndex += 5 * int(d->enumerators.size());
-        dataIndex += 5 * int(d->constructors.size());
+        dataIndex += QMetaObjectPrivate::IntsPerProperty * int(d->properties.size());
+        dataIndex += QMetaObjectPrivate::IntsPerEnum * int(d->enumerators.size());
+        dataIndex += QMetaObjectPrivate::IntsPerMethod * int(d->constructors.size());
     }
 
     // Allocate space for the enumerator key names and values.
@@ -1282,10 +1248,10 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     char *str = reinterpret_cast<char *>(buf + size);
     if (buf) {
         if (relocatable) {
-            meta->d.stringdata = reinterpret_cast<const QByteArrayData *>((quintptr)size);
+            meta->d.stringdata = reinterpret_cast<const uint *>((quintptr)size);
             meta->d.data = reinterpret_cast<uint *>((quintptr)pmetaSize);
         } else {
-            meta->d.stringdata = reinterpret_cast<const QByteArrayData *>(str);
+            meta->d.stringdata = reinterpret_cast<const uint *>(str);
             meta->d.data = reinterpret_cast<uint *>(data);
         }
     }
@@ -1309,6 +1275,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
     // Output the methods in the class.
     Q_ASSERT(!buf || dataIndex == pmeta->methodData);
+    int parameterMetaTypesIndex = int(d->properties.size());
     for (const auto &method : d->methods) {
         int name = strings.enter(method.name());
         int argc = method.parameterCount();
@@ -1320,11 +1287,13 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
             data[dataIndex + 2] = paramsIndex;
             data[dataIndex + 3] = tag;
             data[dataIndex + 4] = attrs;
+            data[dataIndex + 5] = parameterMetaTypesIndex;
             if (method.methodType() == QMetaMethod::Signal)
                 pmeta->signalCount++;
         }
-        dataIndex += 5;
+        dataIndex += QMetaObjectPrivate::IntsPerMethod;
         paramsIndex += 1 + argc * 2;
+        parameterMetaTypesIndex += 1 + argc;
     }
     if (hasRevisionedMethods) {
         for (const auto &method : d->methods) {
@@ -1335,7 +1304,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     }
 
     // Output the method parameters in the class.
-    Q_ASSERT(!buf || dataIndex == pmeta->methodData + int(d->methods.size()) * 5
+    Q_ASSERT(!buf || dataIndex == pmeta->methodData + int(d->methods.size()) * QMetaObjectPrivate::IntsPerMethod
              + (hasRevisionedMethods ? int(d->methods.size()) : 0));
     for (int x = 0; x < 2; ++x) {
         const std::vector<QMetaMethodBuilderPrivate> &methods = (x == 0) ? d->methods : d->constructors;
@@ -1346,7 +1315,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
                 const QByteArray &typeName = (i < 0) ? method.returnType : paramTypeNames.at(i);
                 int typeInfo;
                 if (QtPrivate::isBuiltinType(typeName))
-                    typeInfo = QMetaType::type(typeName);
+                    typeInfo = QMetaType::fromName(typeName).id();
                 else
                     typeInfo = IsUnresolvedType | strings.enter(typeName);
                 if (buf)
@@ -1373,7 +1342,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
         int typeInfo;
         if (QtPrivate::isBuiltinType(prop.type))
-            typeInfo = QMetaType::type(prop.type);
+            typeInfo = QMetaType::fromName(prop.type).id();
         else
             typeInfo = IsUnresolvedType | strings.enter(prop.type);
 
@@ -1386,26 +1355,10 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
             data[dataIndex]     = name;
             data[dataIndex + 1] = typeInfo;
             data[dataIndex + 2] = flags;
+            data[dataIndex + 3] = prop.notifySignal;
+            data[dataIndex + 4] = prop.revision;
         }
-        dataIndex += 3;
-    }
-    if (hasNotifySignals) {
-        for (const auto &prop : d->properties) {
-            if (buf) {
-                if (prop.notifySignal != -1)
-                    data[dataIndex] = prop.notifySignal;
-                else
-                    data[dataIndex] = 0;
-            }
-            ++dataIndex;
-        }
-    }
-    if (hasRevisionedProperties) {
-        for (const auto &prop : d->properties) {
-            if (buf)
-                data[dataIndex] = prop.revision;
-            ++dataIndex;
-        }
+        dataIndex += QMetaObjectPrivate::IntsPerProperty;
     }
 
     // Output the enumerators in the class.
@@ -1431,7 +1384,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
                 data[enumOffset++] = enumerator.values[key];
             }
         }
-        dataIndex += 5;
+        dataIndex += QMetaObjectPrivate::IntsPerEnum;
         enumIndex += 2 * count;
     }
 
@@ -1448,9 +1401,11 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
             data[dataIndex + 2] = paramsIndex;
             data[dataIndex + 3] = tag;
             data[dataIndex + 4] = attrs;
+            data[dataIndex + 5] = parameterMetaTypesIndex;
         }
-        dataIndex += 5;
+        dataIndex += QMetaObjectPrivate::IntsPerMethod;
         paramsIndex += 1 + argc * 2;
+        parameterMetaTypesIndex += argc;
     }
 
     size += strings.blobSize();
@@ -1476,6 +1431,38 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         size += sizeof(SuperData) * (d->relatedMetaObjects.size() + 1);
     }
 
+    if (d->properties.size() > 0 || d->methods.size() > 0 || d->constructors.size() > 0) {
+        ALIGN(size, QtPrivate::QMetaTypeInterface *);
+        auto types = reinterpret_cast<QtPrivate::QMetaTypeInterface **>(buf + size);
+        if (buf) {
+            meta->d.metaTypes = types;
+            for (const auto &prop : d->properties) {
+                QMetaType mt = QMetaType::fromName(prop.type);
+                *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+                types++;
+            }
+            for (const auto &method: d->methods) {
+                QMetaType mt(QMetaType::fromName(method.returnType).id());
+                *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+                types++;
+                for (const auto &parameterType: method.parameterTypes()) {
+                    QMetaType mt = QMetaType::fromName(parameterType);
+                    *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+                    types++;
+                }
+            }
+            for (const auto &constructor : d->constructors) {
+                for (const auto &parameterType : constructor.parameterTypes()) {
+                    QMetaType mt = QMetaType::fromName(parameterType);
+                    *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+                    types++;
+                }
+            }
+        }
+        // parameterMetaTypesIndex is equal to the total number of metatypes
+        size += static_cast<int>(sizeof(QMetaType) * parameterMetaTypesIndex);
+    }
+
     // Align the final size and return it.
     ALIGN(size, void *);
     Q_ASSERT(!buf || size == expectedSize);
@@ -1494,7 +1481,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 */
 QMetaObject *QMetaObjectBuilder::toMetaObject() const
 {
-    int size = buildMetaObject(d, 0, 0, false);
+    int size = buildMetaObject(d, nullptr, 0, false);
     char *buf = reinterpret_cast<char *>(malloc(size));
     memset(buf, 0, size);
     buildMetaObject(d, buf, size, false);
@@ -1517,9 +1504,10 @@ QMetaObject *QMetaObjectBuilder::toMetaObject() const
 */
 QByteArray QMetaObjectBuilder::toRelocatableData(bool *ok) const
 {
-    int size = buildMetaObject(d, 0, 0, true);
+    int size = buildMetaObject(d, nullptr, 0, true);
     if (size == -1) {
-        if (ok) *ok = false;
+        if (ok)
+            *ok = false;
         return QByteArray();
     }
 
@@ -1528,7 +1516,8 @@ QByteArray QMetaObjectBuilder::toRelocatableData(bool *ok) const
     char *buf = data.data();
     memset(buf, 0, size);
     buildMetaObject(d, buf, size, true);
-    if (ok) *ok = true;
+    if (ok)
+        *ok = true;
     return data;
 }
 
@@ -1553,11 +1542,11 @@ void QMetaObjectBuilder::fromRelocatableData(QMetaObject *output,
     quintptr dataOffset = (quintptr)dataMo->d.data;
 
     output->d.superdata = superclass;
-    output->d.stringdata = reinterpret_cast<const QByteArrayData *>(buf + stringdataOffset);
+    output->d.stringdata = reinterpret_cast<const uint *>(buf + stringdataOffset);
     output->d.data = reinterpret_cast<const uint *>(buf + dataOffset);
-    output->d.extradata = 0;
-    output->d.relatedMetaObjects = 0;
-    output->d.static_metacall = 0;
+    output->d.extradata = nullptr;
+    output->d.relatedMetaObjects = nullptr;
+    output->d.static_metacall = nullptr;
 }
 
 /*!
@@ -1598,7 +1587,7 @@ void QMetaObjectBuilder::setStaticMetacallFunction
 
     \sa deserialize()
 */
-void QMetaObjectBuilder::serialize(QDataStream& stream) const
+void QMetaObjectBuilder::serialize(QDataStream &stream) const
 {
     int index;
 
@@ -1610,12 +1599,12 @@ void QMetaObjectBuilder::serialize(QDataStream& stream) const
         stream << QByteArray();
 
     // Write the counts for each type of class member.
-    stream << d->classInfoNames.size();
+    stream << int(d->classInfoNames.size());
     stream << int(d->methods.size());
     stream << int(d->properties.size());
     stream << int(d->enumerators.size());
     stream << int(d->constructors.size());
-    stream << d->relatedMetaObjects.size();
+    stream << int(d->relatedMetaObjects.size());
 
     // Write the items of class information.
     for (index = 0; index < d->classInfoNames.size(); ++index) {
@@ -1640,8 +1629,7 @@ void QMetaObjectBuilder::serialize(QDataStream& stream) const
         stream << property.type;
         stream << property.flags;
         stream << property.notifySignal;
-        if (property.revision)
-            stream << property.revision;
+        stream << property.revision;
     }
 
     // Write the enumerators.
@@ -1675,14 +1663,13 @@ void QMetaObjectBuilder::serialize(QDataStream& stream) const
 }
 
 // Resolve a class name using the name reference map.
-static const QMetaObject *resolveClassName
-        (const QMap<QByteArray, const QMetaObject *>& references,
-         const QByteArray& name)
+static const QMetaObject *resolveClassName(const QMap<QByteArray, const QMetaObject *> &references,
+                                           const QByteArray &name)
 {
     if (name == QByteArray("QObject"))
         return &QObject::staticMetaObject;
     else
-        return references.value(name, 0);
+        return references.value(name, nullptr);
 }
 
 /*!
@@ -1720,14 +1707,14 @@ void QMetaObjectBuilder::deserialize
     d->enumerators.clear();
     d->constructors.clear();
     d->relatedMetaObjects.clear();
-    d->staticMetacallFunction = 0;
+    d->staticMetacallFunction = nullptr;
 
     // Read the class and super class names.
     stream >> d->className;
     stream >> name;
     if (name.isEmpty()) {
-        d->superClass = 0;
-    } else if ((cl = resolveClassName(references, name)) != 0) {
+        d->superClass = nullptr;
+    } else if ((cl = resolveClassName(references, name)) != nullptr) {
         d->superClass = cl;
     } else {
         stream.setStatus(QDataStream::ReadCorruptData);
@@ -1803,8 +1790,7 @@ void QMetaObjectBuilder::deserialize
             stream.setStatus(QDataStream::ReadCorruptData);
             return;
         }
-        if (property.flags & Revisioned)
-            stream >> property.revision;
+        stream >> property.revision;
     }
 
     // Read the enumerators.
@@ -1877,7 +1863,7 @@ QMetaMethodBuilderPrivate *QMetaMethodBuilder::d_func() const
     else if (_mobj && -_index >= 1 && -_index <= int(_mobj->d->constructors.size()))
         return &(_mobj->d->constructors[(-_index) - 1]);
     else
-        return 0;
+        return nullptr;
 }
 
 /*!
@@ -1944,7 +1930,7 @@ QByteArray QMetaMethodBuilder::returnType() const
 
     \sa returnType(), parameterTypes(), signature()
 */
-void QMetaMethodBuilder::setReturnType(const QByteArray& value)
+void QMetaMethodBuilder::setReturnType(const QByteArray &value)
 {
     QMetaMethodBuilderPrivate *d = d_func();
     if (d)
@@ -1984,7 +1970,7 @@ QList<QByteArray> QMetaMethodBuilder::parameterNames() const
 
     \sa parameterNames()
 */
-void QMetaMethodBuilder::setParameterNames(const QList<QByteArray>& value)
+void QMetaMethodBuilder::setParameterNames(const QList<QByteArray> &value)
 {
     QMetaMethodBuilderPrivate *d = d_func();
     if (d)
@@ -2010,7 +1996,7 @@ QByteArray QMetaMethodBuilder::tag() const
 
     \sa setTag()
 */
-void QMetaMethodBuilder::setTag(const QByteArray& value)
+void QMetaMethodBuilder::setTag(const QByteArray &value)
 {
     QMetaMethodBuilderPrivate *d = d_func();
     if (d)
@@ -2084,7 +2070,6 @@ int QMetaMethodBuilder::revision() const
     if (d)
         return d->revision;
     return 0;
-
 }
 
 /*!
@@ -2116,7 +2101,7 @@ QMetaPropertyBuilderPrivate *QMetaPropertyBuilder::d_func() const
     if (_mobj && _index >= 0 && _index < int(_mobj->d->properties.size()))
         return &(_mobj->d->properties[_index]);
     else
-        return 0;
+        return nullptr;
 }
 
 /*!
@@ -2167,7 +2152,7 @@ bool QMetaPropertyBuilder::hasNotifySignal() const
 {
     QMetaPropertyBuilderPrivate *d = d_func();
     if (d)
-        return d->flag(Notify);
+        return d->notifySignal != -1;
     else
         return false;
 }
@@ -2191,16 +2176,14 @@ QMetaMethodBuilder QMetaPropertyBuilder::notifySignal() const
 
     \sa hasNotifySignal(), notifySignal(), removeNotifySignal()
 */
-void QMetaPropertyBuilder::setNotifySignal(const QMetaMethodBuilder& value)
+void QMetaPropertyBuilder::setNotifySignal(const QMetaMethodBuilder &value)
 {
     QMetaPropertyBuilderPrivate *d = d_func();
     if (d) {
         if (value._mobj) {
             d->notifySignal = value._index;
-            d->setFlag(Notify, true);
         } else {
             d->notifySignal = -1;
-            d->setFlag(Notify, false);
         }
     }
 }
@@ -2213,10 +2196,8 @@ void QMetaPropertyBuilder::setNotifySignal(const QMetaMethodBuilder& value)
 void QMetaPropertyBuilder::removeNotifySignal()
 {
     QMetaPropertyBuilderPrivate *d = d_func();
-    if (d) {
+    if (d)
         d->notifySignal = -1;
-        d->setFlag(Notify, false);
-    }
 }
 
 /*!
@@ -2309,21 +2290,6 @@ bool QMetaPropertyBuilder::isStored() const
         return false;
 }
 
-/*!
-    Returns \c true if the property is editable; otherwise returns \c false.
-    This default value is false.
-
-    \sa setEditable(), isDesignable(), isScriptable(), isStored()
-*/
-bool QMetaPropertyBuilder::isEditable() const
-{
-    QMetaPropertyBuilderPrivate *d = d_func();
-    if (d)
-        return d->flag(Editable);
-    else
-        return false;
-}
-
 /*!
     Returns \c true if this property is designated as the \c USER
     property, i.e., the one that the user can edit or that is
@@ -2400,6 +2366,31 @@ bool QMetaPropertyBuilder::isFinal() const
         return false;
 }
 
+/*!
+ * Returns \c true if the property is an alias.
+ * The default value is false
+ */
+bool QMetaPropertyBuilder::isAlias() const
+{
+    QMetaPropertyBuilderPrivate *d = d_func();
+    if (d)
+        return d->flag(Alias);
+    else
+        return false;
+}
+
+/*!
+  Returns \c true if the property is bindable
+  The default value is false
+ */
+bool QMetaPropertyBuilder::isBindable() const
+{
+    if (auto d = d_func())
+        return d->flag(Bindable);
+    else
+        return false;
+}
+
 /*!
     Sets this property to readable if \a value is true.
 
@@ -2472,18 +2463,6 @@ void QMetaPropertyBuilder::setStored(bool value)
         d->setFlag(Stored, value);
 }
 
-/*!
-    Sets this property to editable if \a value is true.
-
-    \sa isEditable(), setDesignable(), setScriptable(), setStored()
-*/
-void QMetaPropertyBuilder::setEditable(bool value)
-{
-    QMetaPropertyBuilderPrivate *d = d_func();
-    if (d)
-        d->setFlag(Editable, value);
-}
-
 /*!
     Sets the \c USER flag on this property to \a value.
 
@@ -2547,6 +2526,25 @@ void QMetaPropertyBuilder::setFinal(bool value)
         d->setFlag(Final, value);
 }
 
+/*!
+   Sets the \c ALIAS flag on this property to \a value
+ */
+void QMetaPropertyBuilder::setAlias(bool value)
+{
+    QMetaPropertyBuilderPrivate *d = d_func();
+    if (d)
+        d->setFlag(Alias, value);
+}
+
+/*!
+   Sets the\c BINDABLE flag on this property to \a value
+ */
+void QMetaPropertyBuilder::setBindable(bool value)
+{
+    if (auto d = d_func())
+        d->setFlag(Bindable, value);
+}
+
 /*!
     Returns the revision of this property.
 
@@ -2558,7 +2556,6 @@ int QMetaPropertyBuilder::revision() const
     if (d)
         return d->revision;
     return 0;
-
 }
 
 /*!
@@ -2569,13 +2566,10 @@ int QMetaPropertyBuilder::revision() const
 void QMetaPropertyBuilder::setRevision(int revision)
 {
     QMetaPropertyBuilderPrivate *d = d_func();
-    if (d) {
+    if (d)
         d->revision = revision;
-        d->setFlag(Revisioned, revision != 0);
-    }
 }
 
-
 /*!
     \class QMetaEnumBuilder
     \inmodule QtCore
@@ -2588,7 +2582,7 @@ QMetaEnumBuilderPrivate *QMetaEnumBuilder::d_func() const
     if (_mobj && _index >= 0 && _index < int(_mobj->d->enumerators.size()))
         return &(_mobj->d->enumerators[_index]);
     else
-        return 0;
+        return nullptr;
 }
 
 /*!
@@ -2743,7 +2737,7 @@ int QMetaEnumBuilder::value(int index) const
 
     \sa keyCount(), key(), value(), removeKey()
 */
-int QMetaEnumBuilder::addKey(const QByteArray& name, int value)
+int QMetaEnumBuilder::addKey(const QByteArray &name, int value)
 {
     QMetaEnumBuilderPrivate *d = d_func();
     if (d) {
