@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtTest module of the Qt Toolkit.
@@ -44,7 +44,7 @@
 #include <QtTest/private/qabstracttestlogger_p.h>
 #include <QtTest/private/qplaintestlogger_p.h>
 #include <QtTest/private/qcsvbenchmarklogger_p.h>
-#include <QtTest/private/qxunittestlogger_p.h>
+#include <QtTest/private/qjunittestlogger_p.h>
 #include <QtTest/private/qxmltestlogger_p.h>
 #include <QtTest/private/qteamcitylogger_p.h>
 #include <QtTest/private/qtaptestlogger_p.h>
@@ -58,8 +58,9 @@
 
 #include <QtCore/qatomic.h>
 #include <QtCore/qbytearray.h>
-#include <QtCore/QElapsedTimer>
-#include <QtCore/QVariant>
+#include <QtCore/qelapsedtimer.h>
+#include <QtCore/qlist.h>
+#include <QtCore/qvariant.h>
 #if QT_CONFIG(regularexpression)
 #include <QtCore/QRegularExpression>
 #endif
@@ -73,6 +74,10 @@ QT_BEGIN_NAMESPACE
 static void saveCoverageTool(const char * appname, bool testfailed, bool installedTestCoverage)
 {
 #ifdef __COVERAGESCANNER__
+#  if QT_CONFIG(testlib_selfcover)
+    __coveragescanner_teststate(QTestLog::failCount() > 0 ? "FAILED" :
+                                QTestLog::passCount() > 0 ? "PASSED" : "SKIPPED");
+#  else
     if (!installedTestCoverage)
         return;
     // install again to make sure the filename is correct.
@@ -83,6 +88,7 @@ static void saveCoverageTool(const char * appname, bool testfailed, bool install
     __coveragescanner_testname("");
     __coveragescanner_clear();
     unsetenv("QT_TESTCOCOON_ACTIVE");
+#  endif // testlib_selfcover
 #else
     Q_UNUSED(appname);
     Q_UNUSED(testfailed);
@@ -93,6 +99,8 @@ static void saveCoverageTool(const char * appname, bool testfailed, bool install
 static QElapsedTimer elapsedFunctionTime;
 static QElapsedTimer elapsedTotalTime;
 
+#define FOREACH_TEST_LOGGER for (QAbstractTestLogger *logger : *QTest::loggers())
+
 namespace QTest {
 
     int fails = 0;
@@ -103,7 +111,7 @@ namespace QTest {
     struct IgnoreResultList
     {
         inline IgnoreResultList(QtMsgType tp, const QVariant &patternIn)
-            : type(tp), pattern(patternIn), next(0) {}
+            : type(tp), pattern(patternIn) {}
 
         static inline void clearList(IgnoreResultList *&list)
         {
@@ -136,7 +144,7 @@ namespace QTest {
             // (the space was added automatically by ~QDebug() until Qt 5.3,
             //  so autotests still might expect it)
             if (expected.endsWith(QLatin1Char(' ')))
-                return actual == expected.leftRef(expected.length() - 1);
+                return actual == QStringView{expected}.left(expected.length() - 1);
 
             return false;
         }
@@ -144,7 +152,7 @@ namespace QTest {
         inline bool matches(QtMsgType tp, const QString &message) const
         {
             return tp == type
-                   && (pattern.type() == QVariant::String ?
+                   && (pattern.userType() == QMetaType::QString ?
                        stringsMatch(pattern.toString(), message) :
 #if QT_CONFIG(regularexpression)
                        pattern.toRegularExpression().match(message).hasMatch());
@@ -155,115 +163,12 @@ namespace QTest {
 
         QtMsgType type;
         QVariant pattern;
-        IgnoreResultList *next;
+        IgnoreResultList *next = nullptr;
     };
 
-    static IgnoreResultList *ignoreResultList = 0;
+    static IgnoreResultList *ignoreResultList = nullptr;
 
-    struct LoggerList
-    {
-        QAbstractTestLogger *logger;
-        LoggerList *next;
-    };
-
-    class TestLoggers
-    {
-    public:
-        static void addLogger(QAbstractTestLogger *logger)
-        {
-            LoggerList *l = new LoggerList;
-            l->logger = logger;
-            l->next = loggers;
-            loggers = l;
-        }
-
-        static void destroyLoggers()
-        {
-            while (loggers) {
-                LoggerList *l = loggers;
-                loggers = loggers->next;
-                delete l->logger;
-                delete l;
-            }
-        }
-
-#define FOREACH_LOGGER(operation) \
-        LoggerList *l = loggers; \
-        while (l) { \
-            QAbstractTestLogger *logger = l->logger; \
-            Q_UNUSED(logger); \
-            operation; \
-            l = l->next; \
-        }
-
-        static void startLogging()
-        {
-            FOREACH_LOGGER(logger->startLogging());
-        }
-
-        static void stopLogging()
-        {
-            FOREACH_LOGGER(logger->stopLogging());
-        }
-
-        static void enterTestFunction(const char *function)
-        {
-            FOREACH_LOGGER(logger->enterTestFunction(function));
-        }
-
-        static void leaveTestFunction()
-        {
-            FOREACH_LOGGER(logger->leaveTestFunction());
-        }
-
-        static void enterTestData(QTestData *data)
-        {
-            FOREACH_LOGGER(logger->enterTestData(data));
-        }
-
-        static void addIncident(QAbstractTestLogger::IncidentTypes type, const char *description,
-                                const char *file = 0, int line = 0)
-        {
-            FOREACH_LOGGER(logger->addIncident(type, description, file, line));
-        }
-
-        static void addBenchmarkResult(const QBenchmarkResult &result)
-        {
-            FOREACH_LOGGER(logger->addBenchmarkResult(result));
-        }
-
-        static void addMessage(QtMsgType type, const QMessageLogContext &context,
-                               const QString &message)
-        {
-            FOREACH_LOGGER(logger->addMessage(type, context, message));
-        }
-
-        static void addMessage(QAbstractTestLogger::MessageTypes type, const QString &message,
-                               const char *file = 0, int line = 0)
-        {
-            FOREACH_LOGGER(logger->addMessage(type, message, file, line));
-        }
-
-        static void outputString(const char *msg)
-        {
-            FOREACH_LOGGER(logger->outputString(msg));
-        }
-
-        static int loggerCount()
-        {
-            int count = 0;
-            FOREACH_LOGGER(++count);
-            return count;
-        }
-
-    private:
-        static LoggerList *loggers;
-    };
-
-#undef FOREACH_LOGGER
-
-    LoggerList *TestLoggers::loggers = 0;
-    static bool loggerUsingStdout = false;
+    Q_GLOBAL_STATIC(QList<QAbstractTestLogger *>, loggers)
 
     static int verbosity = 0;
     static int maxWarnings = 2002;
@@ -275,7 +180,7 @@ namespace QTest {
     {
         if (!ignoreResultList)
             return false;
-        IgnoreResultList *last = 0;
+        IgnoreResultList *last = nullptr;
         IgnoreResultList *list = ignoreResultList;
         while (list) {
             if (list->matches(type, message)) {
@@ -285,7 +190,7 @@ namespace QTest {
                 else if (list->next)
                     ignoreResultList = list->next;
                 else
-                    ignoreResultList = 0;
+                    ignoreResultList = nullptr;
 
                 delete list;
                 return true;
@@ -301,10 +206,10 @@ namespace QTest {
     {
         static QBasicAtomicInt counter = Q_BASIC_ATOMIC_INITIALIZER(QTest::maxWarnings);
 
-        if (QTest::TestLoggers::loggerCount() == 0) {
+        if (QTestLog::loggerCount() == 0) {
             // if this goes wrong, something is seriously broken.
             qInstallMessageHandler(oldMessageHandler);
-            QTEST_ASSERT(QTest::TestLoggers::loggerCount() != 0);
+            QTEST_ASSERT(QTestLog::loggerCount() != 0);
         }
 
         if (handleIgnoredMessage(type, message)) {
@@ -313,17 +218,20 @@ namespace QTest {
         }
 
         if (type != QtFatalMsg) {
-            if (counter.load() <= 0)
+            if (counter.loadRelaxed() <= 0)
                 return;
 
             if (!counter.deref()) {
-                QTest::TestLoggers::addMessage(QAbstractTestLogger::QSystem,
+                FOREACH_TEST_LOGGER {
+                    logger->addMessage(QAbstractTestLogger::QSystem,
                         QStringLiteral("Maximum amount of warnings exceeded. Use -maxwarnings to override."));
+                }
                 return;
             }
         }
 
-        QTest::TestLoggers::addMessage(type, context, message);
+        FOREACH_TEST_LOGGER
+            logger->addMessage(type, context, message);
 
         if (type == QtFatalMsg) {
              /* Right now, we're inside the custom message handler and we're
@@ -346,13 +254,16 @@ void QTestLog::enterTestFunction(const char* function)
 
     QTEST_ASSERT(function);
 
-    QTest::TestLoggers::enterTestFunction(function);
+    FOREACH_TEST_LOGGER
+        logger->enterTestFunction(function);
 }
 
 void QTestLog::enterTestData(QTestData *data)
 {
     QTEST_ASSERT(data);
-    QTest::TestLoggers::enterTestData(data);
+
+    FOREACH_TEST_LOGGER
+        logger->enterTestData(data);
 }
 
 int QTestLog::unhandledIgnoreMessages()
@@ -371,7 +282,8 @@ void QTestLog::leaveTestFunction()
     if (printAvailableTags)
         return;
 
-    QTest::TestLoggers::leaveTestFunction();
+    FOREACH_TEST_LOGGER
+        logger->leaveTestFunction();
 }
 
 void QTestLog::printUnhandledIgnoreMessages()
@@ -379,14 +291,15 @@ void QTestLog::printUnhandledIgnoreMessages()
     QString message;
     QTest::IgnoreResultList *list = QTest::ignoreResultList;
     while (list) {
-        if (list->pattern.type() == QVariant::String) {
+        if (list->pattern.userType() == QMetaType::QString) {
             message = QStringLiteral("Did not receive message: \"") + list->pattern.toString() + QLatin1Char('"');
         } else {
 #if QT_CONFIG(regularexpression)
             message = QStringLiteral("Did not receive any message matching: \"") + list->pattern.toRegularExpression().pattern() + QLatin1Char('"');
 #endif
         }
-        QTest::TestLoggers::addMessage(QAbstractTestLogger::Info, message);
+        FOREACH_TEST_LOGGER
+            logger->addMessage(QAbstractTestLogger::Info, message);
 
         list = list->next;
     }
@@ -406,7 +319,8 @@ void QTestLog::addPass(const char *msg)
 
     ++QTest::passes;
 
-    QTest::TestLoggers::addIncident(QAbstractTestLogger::Pass, msg);
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::Pass, msg);
 }
 
 void QTestLog::addFail(const char *msg, const char *file, int line)
@@ -415,7 +329,8 @@ void QTestLog::addFail(const char *msg, const char *file, int line)
 
     ++QTest::fails;
 
-    QTest::TestLoggers::addIncident(QAbstractTestLogger::Fail, msg, file, line);
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::Fail, msg, file, line);
 }
 
 void QTestLog::addXFail(const char *msg, const char *file, int line)
@@ -423,7 +338,8 @@ void QTestLog::addXFail(const char *msg, const char *file, int line)
     QTEST_ASSERT(msg);
     QTEST_ASSERT(file);
 
-    QTest::TestLoggers::addIncident(QAbstractTestLogger::XFail, msg, file, line);
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::XFail, msg, file, line);
 }
 
 void QTestLog::addXPass(const char *msg, const char *file, int line)
@@ -433,7 +349,8 @@ void QTestLog::addXPass(const char *msg, const char *file, int line)
 
     ++QTest::fails;
 
-    QTest::TestLoggers::addIncident(QAbstractTestLogger::XPass, msg, file, line);
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::XPass, msg, file, line);
 }
 
 void QTestLog::addBPass(const char *msg)
@@ -442,7 +359,8 @@ void QTestLog::addBPass(const char *msg)
 
     ++QTest::blacklists;
 
-    QTest::TestLoggers::addIncident(QAbstractTestLogger::BlacklistedPass, msg);
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::BlacklistedPass, msg);
 }
 
 void QTestLog::addBFail(const char *msg, const char *file, int line)
@@ -452,7 +370,30 @@ void QTestLog::addBFail(const char *msg, const char *file, int line)
 
     ++QTest::blacklists;
 
-    QTest::TestLoggers::addIncident(QAbstractTestLogger::BlacklistedFail, msg, file, line);
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::BlacklistedFail, msg, file, line);
+}
+
+void QTestLog::addBXPass(const char *msg, const char *file, int line)
+{
+    QTEST_ASSERT(msg);
+    QTEST_ASSERT(file);
+
+    ++QTest::blacklists;
+
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::BlacklistedXPass, msg, file, line);
+}
+
+void QTestLog::addBXFail(const char *msg, const char *file, int line)
+{
+    QTEST_ASSERT(msg);
+    QTEST_ASSERT(file);
+
+    ++QTest::blacklists;
+
+    FOREACH_TEST_LOGGER
+        logger->addIncident(QAbstractTestLogger::BlacklistedXFail, msg, file, line);
 }
 
 void QTestLog::addSkip(const char *msg, const char *file, int line)
@@ -462,39 +403,42 @@ void QTestLog::addSkip(const char *msg, const char *file, int line)
 
     ++QTest::skips;
 
-    QTest::TestLoggers::addMessage(QAbstractTestLogger::Skip, QString::fromUtf8(msg), file, line);
+    FOREACH_TEST_LOGGER
+        logger->addMessage(QAbstractTestLogger::Skip, QString::fromUtf8(msg), file, line);
 }
 
 void QTestLog::addBenchmarkResult(const QBenchmarkResult &result)
 {
-    QTest::TestLoggers::addBenchmarkResult(result);
+    FOREACH_TEST_LOGGER
+        logger->addBenchmarkResult(result);
 }
 
 void QTestLog::startLogging()
 {
     elapsedTotalTime.start();
     elapsedFunctionTime.start();
-    QTest::TestLoggers::startLogging();
+    FOREACH_TEST_LOGGER
+        logger->startLogging();
     QTest::oldMessageHandler = qInstallMessageHandler(QTest::messageHandler);
 }
 
 void QTestLog::stopLogging()
 {
     qInstallMessageHandler(QTest::oldMessageHandler);
-    QTest::TestLoggers::stopLogging();
-    QTest::TestLoggers::destroyLoggers();
-    QTest::loggerUsingStdout = false;
+    FOREACH_TEST_LOGGER {
+        logger->stopLogging();
+        delete logger;
+    }
+    QTest::loggers()->clear();
     saveCoverageTool(QTestResult::currentAppName(), failCount() != 0, QTestLog::installedTestCoverage());
 }
 
 void QTestLog::addLogger(LogMode mode, const char *filename)
 {
     if (filename && strcmp(filename, "-") == 0)
-        filename = 0;
-    if (!filename)
-        QTest::loggerUsingStdout = true;
+        filename = nullptr;
 
-    QAbstractTestLogger *logger = 0;
+    QAbstractTestLogger *logger = nullptr;
     switch (mode) {
     case QTestLog::Plain:
         logger = new QPlainTestLogger(filename);
@@ -508,8 +452,8 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
     case QTestLog::LightXML:
         logger = new QXmlTestLogger(QXmlTestLogger::Light, filename);
         break;
-    case QTestLog::XunitXML:
-        logger = new QXunitTestLogger(filename);
+    case QTestLog::JUnitXML:
+        logger = new QJUnitTestLogger(filename);
         break;
     case QTestLog::TeamCity:
         logger = new QTeamCityLogger(filename);
@@ -517,6 +461,11 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
     case QTestLog::TAP:
         logger = new QTapTestLogger(filename);
         break;
+#if defined(QT_USE_APPLE_UNIFIED_LOGGING)
+    case QTestLog::Apple:
+        logger = new QAppleTestLogger;
+        break;
+#endif
 #if defined(HAVE_XCTEST)
     case QTestLog::XCTest:
         logger = new QXcodeTestLogger;
@@ -524,41 +473,53 @@ void QTestLog::addLogger(LogMode mode, const char *filename)
 #endif
     }
 
-#if defined(QT_USE_APPLE_UNIFIED_LOGGING)
-    // Logger that also feeds messages to AUL. It needs to wrap the existing
-    // logger, as it needs to be able to short circuit the existing logger
-    // in case AUL prints to stderr.
-    if (QAppleTestLogger::debugLoggingEnabled())
-        logger = new QAppleTestLogger(logger);
-#endif
-
     QTEST_ASSERT(logger);
-    QTest::TestLoggers::addLogger(logger);
+    addLogger(logger);
+}
+
+/*!
+    \internal
+
+    Adds a new logger to the set of loggers that will be used
+    to report incidents and messages during testing.
+
+    The function takes ownership of the logger.
+*/
+void QTestLog::addLogger(QAbstractTestLogger *logger)
+{
+    QTEST_ASSERT(logger);
+    QTest::loggers()->append(logger);
 }
 
 int QTestLog::loggerCount()
 {
-    return QTest::TestLoggers::loggerCount();
+    return QTest::loggers()->size();
 }
 
 bool QTestLog::loggerUsingStdout()
 {
-    return QTest::loggerUsingStdout;
+    FOREACH_TEST_LOGGER {
+        if (logger->isLoggingToStdout())
+            return true;
+    }
+
+    return false;
 }
 
 void QTestLog::warn(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    if (QTest::TestLoggers::loggerCount() > 0)
-        QTest::TestLoggers::addMessage(QAbstractTestLogger::Warn, QString::fromUtf8(msg), file, line);
+    FOREACH_TEST_LOGGER
+        logger->addMessage(QAbstractTestLogger::Warn, QString::fromUtf8(msg), file, line);
 }
 
 void QTestLog::info(const char *msg, const char *file, int line)
 {
     QTEST_ASSERT(msg);
 
-    QTest::TestLoggers::addMessage(QAbstractTestLogger::Info, QString::fromUtf8(msg), file, line);
+    FOREACH_TEST_LOGGER
+        logger->addMessage(QAbstractTestLogger::Info, QString::fromUtf8(msg), file, line);
 }
 
 void QTestLog::setVerboseLevel(int level)
@@ -575,7 +536,7 @@ void QTestLog::ignoreMessage(QtMsgType type, const char *msg)
 {
     QTEST_ASSERT(msg);
 
-    QTest::IgnoreResultList::append(QTest::ignoreResultList, type, QString::fromLocal8Bit(msg));
+    QTest::IgnoreResultList::append(QTest::ignoreResultList, type, QString::fromUtf8(msg));
 }
 
 #if QT_CONFIG(regularexpression)
@@ -652,3 +613,5 @@ qint64 QTestLog::nsecsFunctionTime()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qtestlog_p.cpp"
